@@ -1,37 +1,18 @@
 /*
- * Copyright 1990 - 1994, Julianne Frances Haugh
- * All rights reserved.
+ * SPDX-FileCopyrightText: 1990 - 1994, Julianne Frances Haugh
+ * SPDX-FileCopyrightText: 1996 - 1998, Marek Michałkiewicz
+ * SPDX-FileCopyrightText: 2005       , Tomasz Kłoczko
+ * SPDX-FileCopyrightText: 2008       , Nicolas François
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. Neither the name of Julianne F. Haugh nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY JULIE HAUGH AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL JULIE HAUGH OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
+ * SPDX-License-Identifier: BSD-3-Clause
  */
 
 #include <config.h>
 
-#ident "$Id: sgetgrent.c 1657 2008-01-06 11:59:01Z nekral-guest $"
+#ident "$Id$"
 
 #include <stdio.h>
+#include <sys/types.h>
 #include <grp.h>
 #include "defines.h"
 #include "prototypes.h"
@@ -84,10 +65,12 @@ static char **list (char *s)
 		if (!s || s[0] == '\0')
 			break;
 		members[i++] = s;
-		while (*s && *s != ',')
+		while (('\0' != *s) && (',' != *s)) {
 			s++;
-		if (*s)
+		}
+		if ('\0' != *s) {
 			*s++ = '\0';
+		}
 	}
 	members[i] = (char *) 0;
 	return members;
@@ -117,22 +100,32 @@ struct group *sgetgrent (const char *buf)
 	}
 	strcpy (grpbuf, buf);
 
-	if ((cp = strrchr (grpbuf, '\n')))
+	cp = strrchr (grpbuf, '\n');
+	if (NULL != cp) {
 		*cp = '\0';
-
-	for (cp = grpbuf, i = 0; i < NFIELDS && cp; i++) {
-		grpfields[i] = cp;
-		if ((cp = strchr (cp, ':')))
-			*cp++ = 0;
 	}
-	if (i < (NFIELDS - 1) || *grpfields[2] == '\0')
-		return 0;
+
+	for (cp = grpbuf, i = 0; (i < NFIELDS) && (NULL != cp); i++) {
+		grpfields[i] = cp;
+		cp = strchr (cp, ':');
+		if (NULL != cp) {
+			*cp = '\0';
+			cp++;
+		}
+	}
+	if (i < (NFIELDS - 1) || *grpfields[2] == '\0' || cp != NULL) {
+		return (struct group *) 0;
+	}
 	grent.gr_name = grpfields[0];
 	grent.gr_passwd = grpfields[1];
-	grent.gr_gid = atoi (grpfields[2]);
+	if (get_gid (grpfields[2], &grent.gr_gid) == 0) {
+		return (struct group *) 0;
+	}
 	grent.gr_mem = list (grpfields[3]);
-	if (!grent.gr_mem)
+	if (NULL == grent.gr_mem) {
 		return (struct group *) 0;	/* out of memory */
+	}
 
 	return &grent;
 }
+
