@@ -1,38 +1,15 @@
 /*
- * Copyright (c) 1989 - 1994, Julianne Frances Haugh
- * Copyright (c) 1996 - 2001, Marek Michałkiewicz
- * Copyright (c) 2001 - 2006, Tomasz Kłoczko
- * Copyright (c) 2007 - 2012, Nicolas François
- * All rights reserved.
+ * SPDX-FileCopyrightText: 1989 - 1994, Julianne Frances Haugh
+ * SPDX-FileCopyrightText: 1996 - 2001, Marek Michałkiewicz
+ * SPDX-FileCopyrightText: 2001 - 2006, Tomasz Kłoczko
+ * SPDX-FileCopyrightText: 2007 - 2012, Nicolas François
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. The name of the copyright holders or contributors may not be used to
- *    endorse or promote products derived from this software without
- *    specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
- * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
- * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * SPDX-License-Identifier: BSD-3-Clause
  */
 
 #include <config.h>
 
-#ident "$Id: login.c 3743 2012-05-25 11:51:53Z nekral-guest $"
+#ident "$Id$"
 
 #include <errno.h>
 #include <grp.h>
@@ -53,6 +30,7 @@
 #include "pwauth.h"
 /*@-exitarg@*/
 #include "exitcodes.h"
+#include "shadowlog.h"
 
 #ifdef USE_PAM
 #include "pam_defs.h"
@@ -103,7 +81,7 @@ static bool hflg = false;
 static bool preauth_flag = false;
 
 static bool amroot;
-static unsigned int timeout;
+static char tmsg[256];
 
 /*
  * External identifiers.
@@ -129,7 +107,12 @@ static /*@observer@*/const char *get_failent_user (/*@returned@*/const char *use
 static void update_utmp (const char *user,
                          const char *tty,
                          const char *host,
-                         /*@null@*/const struct utmp *utent);
+#ifdef USE_UTMPX
+                         /*@null@*/const struct utmpx *utent
+#else
+                         /*@null@*/const struct utmp *utent
+#endif
+			);
 
 #ifndef USE_PAM
 static struct faillog faillog;
@@ -211,7 +194,7 @@ static void setup_tty (void)
 
 		/*
 		 * ttymon invocation prefers this, but these settings
-		 * won't come into effect after the first username login 
+		 * won't come into effect after the first username login
 		 */
 		(void) STTY (0, &termio);
 	}
@@ -230,7 +213,7 @@ static void bad_time_notify (void)
 
 static void check_nologin (bool login_to_root)
 {
-	char *fname;
+	const char *fname;
 
 	/*
 	 * Check to see if system is turned off for non-root users.
@@ -370,7 +353,7 @@ static void process_flags (int argc, char *const *argv)
 static void init_env (void)
 {
 #ifndef USE_PAM
-	char *cp;
+	const char *cp;
 #endif
 	char *tmp;
 
@@ -395,7 +378,7 @@ static void init_env (void)
 		}
 	}
 #endif				/* !USE_PAM */
-	/* 
+	/*
 	 * Add the clock frequency so that profiling commands work
 	 * correctly.
 	 */
@@ -416,8 +399,8 @@ static void init_env (void)
 
 static RETSIGTYPE alarm_handler (unused int sig)
 {
-	fprintf (stderr, _("\nLogin timed out after %u seconds.\n"), timeout);
-	exit (0);
+	write (STDERR_FILENO, tmsg, strlen (tmsg));
+	_exit (0);
 }
 
 #ifdef USE_PAM
@@ -481,17 +464,23 @@ static /*@observer@*/const char *get_failent_user (/*@returned@*/const char *use
 static void update_utmp (const char *user,
                          const char *tty,
                          const char *host,
-                         /*@null@*/const struct utmp *utent)
+#ifdef USE_UTMPX
+                         /*@null@*/const struct utmpx *utent
+#else
+                         /*@null@*/const struct utmp *utent
+#endif
+			 )
 {
-	struct utmp  *ut  = prepare_utmp  (user, tty, host, utent);
 #ifdef USE_UTMPX
 	struct utmpx *utx = prepare_utmpx (user, tty, host, utent);
+#else
+	struct utmp  *ut  = prepare_utmp  (user, tty, host, utent);
 #endif				/* USE_UTMPX */
 
+#ifndef USE_UTMPX
 	(void) setutmp  (ut);	/* make entry in the utmp & wtmp files */
 	free (ut);
-
-#ifdef USE_UTMPX
+#else
 	(void) setutmpx (utx);	/* make entry in the utmpx & wtmpx files */
 	free (utx);
 #endif				/* USE_UTMPX */
@@ -508,7 +497,7 @@ static void update_utmp (const char *user,
  *	of reasons, such as X servers or network logins.
  *
  *	the flags which login supports are
- *	
+ *
  *	-p - preserve the environment
  *	-r - perform autologin protocol for rlogin
  *	-f - do not perform authentication, user is preauthenticated
@@ -522,7 +511,7 @@ int main (int argc, char **argv)
 #ifdef RLOGIN
 	char term[128] = "";
 #endif				/* RLOGIN */
-#if defined(HAVE_STRFTIME) && !defined(USE_PAM)
+#if !defined(USE_PAM)
 	char ptime[80];
 #endif
 	unsigned int delay;
@@ -532,13 +521,18 @@ int main (int argc, char **argv)
 	bool is_console;
 #endif
 	int err;
+	unsigned int timeout;
 	const char *cp;
 	const char *tmp;
 	char fromhost[512];
 	struct passwd *pwd = NULL;
 	char **envp = environ;
 	const char *failent_user;
+#ifdef USE_UTMPX
+	/*@null@*/struct utmpx *utent;
+#else
 	/*@null@*/struct utmp *utent;
+#endif
 
 #ifdef USE_PAM
 	int retcode;
@@ -561,6 +555,8 @@ int main (int argc, char **argv)
 
 	amroot = (getuid () == 0);
 	Prog = Basename (argv[0]);
+	log_set_progname(Prog);
+	log_set_logfd(stderr);
 
 	if (geteuid() != 0) {
 		fprintf (stderr, _("%s: Cannot possibly work without effective root\n"), Prog);
@@ -632,7 +628,7 @@ int main (int argc, char **argv)
 	(void) umask (getdef_num ("UMASK", GETDEF_DEFAULT_UMASK));
 
 	{
-		/* 
+		/*
 		 * Use the ULIMIT in the login.defs file, and if
 		 * there isn't one, use the default value. The
 		 * user may have one for themselves, but otherwise,
@@ -680,7 +676,7 @@ int main (int argc, char **argv)
 
 	if (rflg || hflg) {
 		cp = hostname;
-#ifdef	HAVE_STRUCT_UTMP_UT_HOST
+#if defined(HAVE_STRUCT_UTMP_UT_HOST) || defined(USE_UTMPX)
 	} else if ((NULL != utent) && ('\0' != utent->ut_host[0])) {
 		cp = utent->ut_host;
 #endif				/* HAVE_STRUCT_UTMP_UT_HOST */
@@ -698,8 +694,10 @@ int main (int argc, char **argv)
 
       top:
 	/* only allow ALARM sec. for login */
-	(void) signal (SIGALRM, alarm_handler);
 	timeout = getdef_unum ("LOGIN_TIMEOUT", ALARM);
+	snprintf (tmsg, sizeof tmsg,
+	          _("\nLogin timed out after %u seconds.\n"), timeout);
+	(void) signal (SIGALRM, alarm_handler);
 	if (timeout > 0) {
 		(void) alarm (timeout);
 	}
@@ -960,6 +958,19 @@ int main (int argc, char **argv)
 			    || ('*' == user_passwd[0])) {
 				failed = true;
 			}
+
+			if (strcmp (user_passwd, "") == 0) {
+				char *prevent_no_auth = getdef_str("PREVENT_NO_AUTH");
+				if (prevent_no_auth == NULL) {
+					prevent_no_auth = "superuser";
+				}
+				if (strcmp(prevent_no_auth, "yes") == 0) {
+					failed = true;
+				} else if ((pwd->pw_uid == 0)
+					&& (strcmp(prevent_no_auth, "superuser") == 0)) {
+					failed = true;
+				}
+			}
 		}
 
 		if (strcmp (user_passwd, SHADOW_PASSWD_STRING) == 0) {
@@ -1144,7 +1155,9 @@ int main (int argc, char **argv)
 #endif				/* WITH_AUDIT */
 
 #ifndef USE_PAM			/* pam_lastlog handles this */
-	if (getdef_bool ("LASTLOG_ENAB")) {	/* give last login and log this one */
+	if (   getdef_bool ("LASTLOG_ENAB")
+	    && pwd->pw_uid <= (uid_t) getdef_ulong ("LASTLOG_UID_MAX", 0xFFFFFFFFUL)) {
+		/* give last login and log this one */
 		dolastlog (&ll, pwd, tty, hostname);
 	}
 #endif
@@ -1160,7 +1173,7 @@ int main (int argc, char **argv)
 			 * entries.
 			 * Use the x variants because we need to keep the
 			 * entry for a long time, and there might be other
-			 * getxxyy in between.
+			 * getxxyyy in between.
 			 */
 			pw_free (pwd);
 			pwd = xgetpwnam (username);
@@ -1253,6 +1266,7 @@ int main (int argc, char **argv)
 			env++;
 		}
 	}
+	(void) pam_end (pamh, PAM_SUCCESS | PAM_DATA_SILENT);
 #endif
 
 	(void) setlocale (LC_ALL, "");
@@ -1280,19 +1294,15 @@ int main (int argc, char **argv)
 			}
 		}
 		if (   getdef_bool ("LASTLOG_ENAB")
+		    && pwd->pw_uid <= (uid_t) getdef_ulong ("LASTLOG_UID_MAX", 0xFFFFFFFFUL)
 		    && (ll.ll_time != 0)) {
 			time_t ll_time = ll.ll_time;
 
-#ifdef HAVE_STRFTIME
 			(void) strftime (ptime, sizeof (ptime),
 			                 "%a %b %e %H:%M:%S %z %Y",
 			                 localtime (&ll_time));
 			printf (_("Last login: %s on %s"),
 			        ptime, ll.ll_line);
-#else
-			printf (_("Last login: %.19s on %s"),
-			        ctime (&ll_time), ll.ll_line);
-#endif
 #ifdef HAVE_LL_HOST		/* __linux__ || SUN4 */
 			if ('\0' != ll.ll_host[0]) {
 				printf (_(" from %.*s"),
