@@ -1,4 +1,9 @@
 /* Taken from logdaemon-5.0, only minimal changes.  --marekm */
+/*
+ * SPDX-FileCopyrightText: 1990 - 1995, Wietse Venema.
+ *
+ * SPDX-License-Identifier: Unlicense
+ */
 
 /************************************************************************
 * Copyright 1995 by Wietse Venema.  All rights reserved. Individual files
@@ -10,7 +15,7 @@
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this entire copyright notice is duplicated in all such
-* copies.  
+* copies.
 *
 * This software is provided "as is" and without any expressed or implied
 * warranties, including, without limitation, the implied warranties of
@@ -22,7 +27,7 @@
 #endif
 
 #ifndef USE_PAM
-#ident "$Id: login_nopam.c 3182 2010-03-23 11:26:34Z nekral-guest $"
+#ident "$Id$"
 
 #include "prototypes.h"
     /*
@@ -30,7 +35,7 @@
      * control based on login names and on host (or domain) names, internet
      * addresses (or network numbers), or on terminal line names in case of
      * non-networked logins. Diagnostics are reported through syslog(3).
-     * 
+     *
      * Author: Wietse Venema, Eindhoven University of Technology, The Netherlands.
      */
 #include <sys/types.h>
