@@ -1,38 +1,15 @@
 /*
- * Copyright (c) 1990 - 1994, Julianne Frances Haugh
- * Copyright (c) 1996 - 2000, Marek Michałkiewicz
- * Copyright (c) 2001 - 2006, Tomasz Kłoczko
- * Copyright (c) 2007 - 2008, Nicolas François
- * All rights reserved.
+ * SPDX-FileCopyrightText: 1990 - 1994, Julianne Frances Haugh
+ * SPDX-FileCopyrightText: 1996 - 2000, Marek Michałkiewicz
+ * SPDX-FileCopyrightText: 2001 - 2006, Tomasz Kłoczko
+ * SPDX-FileCopyrightText: 2007 - 2008, Nicolas François
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. The name of the copyright holders or contributors may not be used to
- *    endorse or promote products derived from this software without
- *    specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
- * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
- * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * SPDX-License-Identifier: BSD-3-Clause
  */
 
 #include <config.h>
 
-#ident "$Id: newgrp.c 2982 2009-05-22 11:10:02Z nekral-guest $"
+#ident "$Id$"
 
 #include <errno.h>
 #include <grp.h>
@@ -44,11 +21,12 @@
 #include "prototypes.h"
 /*@-exitarg@*/
 #include "exitcodes.h"
+#include "shadowlog.h"
 
 /*
  * Global variables
  */
-char *Prog;
+const char *Prog;
 
 extern char **newenvp;
 extern char **environ;
@@ -83,16 +61,30 @@ static void usage (void)
 	}
 }
 
-/*
- * find_matching_group - search all groups of a given group id for
- *                       membership of a given username
- */
-static /*@null@*/struct group *find_matching_group (const char *name, gid_t gid)
+static bool ingroup(const char *name, struct group *gr)
 {
-	struct group *gr;
 	char **look;
 	bool notfound = true;
 
+	look = gr->gr_mem;
+	while (*look && notfound)
+		notfound = strcmp (*look++, name);
+
+	return !notfound;
+}
+
+/*
+ * find_matching_group - search all groups of a gr's group id for
+ *                       membership of a given username
+ *                       but check gr itself first
+ */
+static /*@null@*/struct group *find_matching_group (const char *name, struct group *gr)
+{
+	gid_t gid = gr->gr_gid;
+
+	if (ingroup(name, gr))
+		return gr;
+
 	setgrent ();
 	while ((gr = getgrent ()) != NULL) {
 		if (gr->gr_gid != gid) {
@@ -103,14 +95,8 @@ static /*@null@*/struct group *find_matching_group (const char *name, gid_t gid)
 		 * A group with matching GID was found.
 		 * Test for membership of 'name'.
 		 */
-		look = gr->gr_mem;
-		while ((NULL != *look) && notfound) {
-			notfound = (strcmp (*look, name) != 0);
-			look++;
-		}
-		if (!notfound) {
+		if (ingroup(name, gr))
 			break;
-		}
 	}
 	endgrent ();
 	return gr;
@@ -153,7 +139,8 @@ static void check_perms (const struct group *grp,
 	 */
 	spwd = xgetspnam (pwd->pw_name);
 	if (NULL != spwd) {
-		pwd->pw_passwd = spwd->sp_pwdp;
+		pwd->pw_passwd = xstrdup (spwd->sp_pwdp);
+		spw_free (spwd);
 	}
 
 	if ((pwd->pw_passwd[0] == '\0') && (grp->gr_passwd[0] != '\0')) {
@@ -184,6 +171,16 @@ static void check_perms (const struct group *grp,
 		cpasswd = pw_encrypt (cp, grp->gr_passwd);
 		strzero (cp);
 
+		if (NULL == cpasswd) {
+			fprintf (stderr,
+			         _("%s: failed to crypt password with previous salt: %s\n"),
+			         Prog, strerror (errno));
+			SYSLOG ((LOG_INFO,
+			         "Failed to crypt password with previous salt of group '%s'",
+			         groupname));
+			goto failure;
+		}
+
 		if (grp->gr_passwd[0] == '\0' ||
 		    strcmp (cpasswd, grp->gr_passwd) != 0) {
 #ifdef WITH_AUDIT
@@ -238,19 +235,22 @@ failure:
 /*
  * syslog_sg - log the change of group to syslog
  *
- *	The loggout will also be logged when the user will quit the
+ *	The logout will also be logged when the user will quit the
  *	sg/newgrp session.
  */
 static void syslog_sg (const char *name, const char *group)
 {
 	const char *loginname = getlogin ();
 	const char *tty = ttyname (0);
+	char *free_login = NULL, *free_tty = NULL;
 
 	if (loginname != NULL) {
-		loginname = xstrdup (loginname);
+		free_login = xstrdup (loginname);
+		loginname = free_login;
 	}
 	if (tty != NULL) {
-		tty = xstrdup (tty);
+		free_tty = xstrdup (tty);
+		tty = free_tty;
 	}
 
 	if (loginname == NULL) {
@@ -282,7 +282,7 @@ static void syslog_sg (const char *name, const char *group)
 	 * receives SIGCHLD from the terminating subshell.  -- JWP
 	 */
 	{
-		pid_t child, pid;
+		pid_t child;
 
 		/* Ignore these signals. The signal handlers will later be
 		 * restored to the default handlers. */
@@ -316,13 +316,15 @@ static void syslog_sg (const char *name, const char *group)
 			int cst = 0;
 			gid_t gid = getgid();
 			struct group *grp = getgrgid (gid);
+			pid_t pid;
 
 			do {
 				errno = 0;
 				pid = waitpid (child, &cst, WUNTRACED);
 				if ((pid == child) && (WIFSTOPPED (cst) != 0)) {
-					/* stop when child stops */
-					kill (getpid (), WSTOPSIG(cst));
+					/* The child (shell) was suspended.
+					 * Suspend sg/newgrp. */
+					kill (getpid (), SIGSTOP);
 					/* wake child when resumed */
 					kill (child, SIGCONT);
 				}
@@ -360,6 +362,8 @@ static void syslog_sg (const char *name, const char *group)
 		(void) signal (SIGTTOU, SIG_DFL);
 	}
 #endif				/* USE_PAM */
+	free(free_login);
+	free(free_tty);
 }
 #endif				/* USE_SYSLOG */
 
@@ -370,10 +374,12 @@ int main (int argc, char **argv)
 {
 	bool initflag = false;
 	int i;
+	bool is_member = false;
 	bool cflag = false;
 	int err = 0;
 	gid_t gid;
 	char *cp;
+	char *progbase;
 	const char *name, *prog;
 	char *group = NULL;
 	char *command = NULL;
@@ -416,9 +422,10 @@ int main (int argc, char **argv)
 	 * don't need to re-exec anything.  -- JWP
 	 */
 	Prog = Basename (argv[0]);
+	log_set_progname(Prog);
+	log_set_logfd(stderr);
 	is_newgrp = (strcmp (Prog, "newgrp") == 0);
 	OPENLOG (is_newgrp ? "newgrp" : "sg");
-	gid = getgid ();
 	argc--;
 	argv++;
 
@@ -465,7 +472,7 @@ int main (int argc, char **argv)
 		initflag = true;
 	}
 	if (!is_newgrp) {
-		/* 
+		/*
 		 * Do the command line for everything that is
 		 * not "newgrp".
 		 */
@@ -504,7 +511,7 @@ int main (int argc, char **argv)
 			group = argv[0];
 		} else {
 			/*
-			 * get the group file entry for her login group id. 
+			 * get the group file entry for her login group id.
 			 * the entry must exist, simply to be annoying.
 			 *
 			 * Perhaps in the past, but the default behavior now depends on the
@@ -566,17 +573,17 @@ int main (int argc, char **argv)
 	 * now we put her in the new group. The password file entry for her
 	 * current user id has been gotten. If there was no optional group
 	 * argument she will have her real and effective group id set to the
-	 * set to the value from her password file entry.  
+	 * set to the value from her password file entry.
 	 *
 	 * If run as newgrp, or as sg with no command, this process exec's
-	 * an interactive subshell with the effective GID of the new group. 
+	 * an interactive subshell with the effective GID of the new group.
 	 * If run as sg with a command, that command is exec'ed in this
 	 * subshell. When this process terminates, either because the user
 	 * exits, or the command completes, the parent of this process
 	 * resumes with the current GID.
 	 *
 	 * If a group is explicitly specified on the command line, the
-	 * interactive shell or command is run with that effective GID. 
+	 * interactive shell or command is run with that effective GID.
 	 * Access will be denied if no entry for that group can be found in
 	 * /etc/group. If the current user name appears in the members list
 	 * for that group, access will be granted immediately; if not, the
@@ -608,22 +615,36 @@ int main (int argc, char **argv)
 		goto failure;
 	}
 
+#ifdef HAVE_SETGROUPS
+	/* when using pam_group, she will not be listed in the groups
+	 * database. However getgroups() will return the group. So
+	 * if she is listed there already it is ok to grant membership.
+	 */
+	for (i = 0; i < ngroups; i++) {
+		if (grp->gr_gid == grouplist[i]) {
+			is_member = true;
+			break;
+		}
+	}
+#endif                          /* HAVE_SETGROUPS */
 	/*
-	 * For splitted groups (due to limitations of NIS), check all 
+	 * For splitted groups (due to limitations of NIS), check all
 	 * groups of the same GID like the requested group for
 	 * membership of the current user.
 	 */
-	grp = find_matching_group (name, grp->gr_gid);
-	if (NULL == grp) {
-		/*
-		 * No matching group found. As we already know that
-		 * the group exists, this happens only in the case
-		 * of a requested group where the user is not member.
-		 *
-		 * Re-read the group entry for further processing.
-		 */
-		grp = xgetgrnam (group);
-		assert (NULL != grp);
+	if (!is_member) {
+		grp = find_matching_group (name, grp);
+		if (NULL == grp) {
+			/*
+			 * No matching group found. As we already know that
+			 * the group exists, this happens only in the case
+			 * of a requested group where the user is not member.
+			 *
+			 * Re-read the group entry for further processing.
+			 */
+			grp = xgetgrnam (group);
+			assert (NULL != grp);
+		}
 	}
 #ifdef SHADOWGRP
 	sgrp = getsgnam (group);
@@ -636,7 +657,9 @@ int main (int argc, char **argv)
 	/*
 	 * Check if the user is allowed to access this group.
 	 */
-	check_perms (grp, pwd, group);
+	if (!is_member) {
+		check_perms (grp, pwd, group);
+	}
 
 	/*
 	 * all successful validations pass through this point. The group id
@@ -653,7 +676,7 @@ int main (int argc, char **argv)
 #ifdef HAVE_SETGROUPS
 	/*
 	 * I am going to try to add her new group id to her concurrent group
-	 * set. If the group id is already present i'll just skip this part. 
+	 * set. If the group id is already present i'll just skip this part.
 	 * If the group doesn't fit, i'll complain loudly and skip this
 	 * part.
 	 */
@@ -722,7 +745,7 @@ int main (int argc, char **argv)
 	 */
 	if (cflag) {
 		closelog ();
-		execl ("/bin/sh", "sh", "-c", command, (char *) 0);
+		execl (SHELL, "sh", "-c", command, (char *) 0);
 #ifdef WITH_AUDIT
 		snprintf (audit_buf, sizeof(audit_buf),
 		          "changing new-gid=%lu", (unsigned long) gid);
@@ -730,7 +753,7 @@ int main (int argc, char **argv)
 		              audit_buf, NULL,
 		              (unsigned int) getuid (), 0);
 #endif
-		perror ("/bin/sh");
+		perror (SHELL);
 		exit ((errno == ENOENT) ? E_CMD_NOTFOUND : E_CMD_NOEXEC);
 	}
 
@@ -757,14 +780,14 @@ int main (int argc, char **argv)
 	} else if ((NULL != pwd->pw_shell) && ('\0' != pwd->pw_shell[0])) {
 		prog = pwd->pw_shell;
 	} else {
-		prog = "/bin/sh";
+		prog = SHELL;
 	}
 
 	/*
 	 * Now I try to find the basename of the login shell. This will
 	 * become argv[0] of the spawned command.
 	 */
-	cp = Basename ((char *) prog);
+	progbase = (char *) Basename ((char *) prog);
 
 	/*
 	 * Switch back to her home directory if i am doing login
@@ -802,14 +825,14 @@ int main (int argc, char **argv)
 	 * Exec the login shell and go away. We are trying to get back to
 	 * the previous environment which should be the user's login shell.
 	 */
-	err = shell (prog, initflag ? (char *) 0 : cp, newenvp);
+	err = shell (prog, initflag ? (char *) 0 : progbase, newenvp);
 	exit ((err == ENOENT) ? E_CMD_NOTFOUND : E_CMD_NOEXEC);
-	/* @notreached@ */
+	/*@notreached@*/
       failure:
 
 	/*
 	 * The previous code, when run as newgrp, re-exec'ed the shell in
-	 * the current process with the original gid on error conditions. 
+	 * the current process with the original gid on error conditions.
 	 * See the comment above. This historical behavior now has the
 	 * effect of creating unlogged extraneous shell layers when the
 	 * command line has an error or there is an authentication failure.
@@ -822,7 +845,7 @@ int main (int argc, char **argv)
 	if (NULL != group) {
 		snprintf (audit_buf, sizeof(audit_buf),
 		          "changing new-group=%s", group);
-		audit_logger (AUDIT_CHGRP_ID, Prog, 
+		audit_logger (AUDIT_CHGRP_ID, Prog,
 		              audit_buf, NULL,
 		              (unsigned int) getuid (), 0);
 	} else {
