@@ -1,31 +1,18 @@
 /*
   vipw, vigr  edit the password or group file
   with -s will edit shadow or gshadow file
-
-  Copyright (c) 1997       , Guy Maor <maor@ece.utexas.edu>
-  Copyright (c) 1999 - 2000, Marek Michałkiewicz
-  Copyright (c) 2002 - 2006, Tomasz Kłoczko
-  Copyright (c) 2007 - 2011, Nicolas François
-  All rights reserved.
-
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 51 Franklin Street, Fifth Floor,
-  Boston, MA 02110-1301, USA.  */
+ *
+ * SPDX-FileCopyrightText: 1997       , Guy Maor <maor@ece.utexas.edu>
+ * SPDX-FileCopyrightText: 1999 - 2000, Marek Michałkiewicz
+ * SPDX-FileCopyrightText: 2002 - 2006, Tomasz Kłoczko
+ * SPDX-FileCopyrightText: 2007 - 2013, Nicolas François
+ *
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ */
 
 #include <config.h>
 
-#ident "$Id: vipw.c 3654 2011-12-09 21:35:57Z nekral-guest $"
+#ident "$Id$"
 
 #include <errno.h>
 #include <getopt.h>
@@ -42,6 +29,7 @@
 #include "defines.h"
 #include "groupio.h"
 #include "nscd.h"
+#include "sssd.h"
 #include "prototypes.h"
 #include "pwio.h"
 #include "sgroupio.h"
@@ -52,6 +40,7 @@
 #include <tcb.h>
 #include "tcbfuncs.h"
 #endif				/* WITH_TCB */
+#include "shadowlog.h"
 
 #define MSG_WARN_EDIT_OTHER_FILE _( \
 	"You have modified %s.\n"\
@@ -180,7 +169,10 @@ static void vipwexit (const char *msg, int syserr, int ret)
 	if (0 != syserr) {
 		fprintf (stderr, ": %s", strerror (err));
 	}
-	(void) fputs ("\n", stderr);
+	if (   (NULL != msg)
+	    || (0 != syserr)) {
+		(void) fputs ("\n", stderr);
+	}
 	if (!quiet) {
 		fprintf (stdout, _("%s: %s is unchanged\n"), Prog,
 			 filename);
@@ -203,6 +195,8 @@ vipwedit (const char *file, int (*file_lock) (void), int (*file_unlock) (void))
 	struct stat st1, st2;
 	int status;
 	FILE *f;
+	pid_t orig_pgrp, editor_pgrp = -1;
+	sigset_t mask, omask;
 	/* FIXME: the following should have variable sizes */
 	char filebackup[1024], fileedit[1024];
 	char *to_rename;
@@ -237,13 +231,13 @@ vipwedit (const char *file, int (*file_lock) (void), int (*file_unlock) (void))
 	/* if SE Linux is enabled then set the context of all new files
 	   to be the context of the file we are editing */
 	if (is_selinux_enabled () != 0) {
-		security_context_t passwd_context=NULL;
+		char *passwd_context_raw = NULL;
 		int ret = 0;
-		if (getfilecon (file, &passwd_context) < 0) {
+		if (getfilecon_raw (file, &passwd_context_raw) < 0) {
 			vipwexit (_("Couldn't get file context"), errno, 1);
 		}
-		ret = setfscreatecon (passwd_context);
-		freecon (passwd_context);
+		ret = setfscreatecon_raw (passwd_context_raw);
+		freecon (passwd_context_raw);
 		if (0 != ret) {
 			vipwexit (_("setfscreatecon () failed"), errno, 1);
 		}
@@ -290,6 +284,8 @@ vipwedit (const char *file, int (*file_lock) (void), int (*file_unlock) (void))
 		editor = DEFAULT_EDITOR;
 	}
 
+	orig_pgrp = tcgetpgrp(STDIN_FILENO);
+
 	pid = fork ();
 	if (-1 == pid) {
 		vipwexit ("fork", 1, 1);
@@ -297,36 +293,94 @@ vipwedit (const char *file, int (*file_lock) (void), int (*file_unlock) (void))
 		/* use the system() call to invoke the editor so that it accepts
 		   command line args in the EDITOR and VISUAL environment vars */
 		char *buf;
+		int status;
+
+		/* Wait for parent to make us the foreground pgrp. */
+		if (orig_pgrp != -1) {
+			pid = getpid();
+			setpgid(0, 0);
+			while (tcgetpgrp(STDIN_FILENO) != pid)
+				continue;
+		}
 
 		buf = (char *) malloc (strlen (editor) + strlen (fileedit) + 2);
 		snprintf (buf, strlen (editor) + strlen (fileedit) + 2,
-			  "%s %s", editor, fileedit);
-		if (system (buf) != 0) {
-			fprintf (stderr, "%s: %s: %s\n", Prog, editor,
+		          "%s %s", editor, fileedit);
+		status = system (buf);
+		if (-1 == status) {
+			fprintf (stderr, _("%s: %s: %s\n"), Prog, editor,
 			         strerror (errno));
 			exit (1);
+		} else if (   WIFEXITED (status)
+		           && (WEXITSTATUS (status) != 0)) {
+			fprintf (stderr, _("%s: %s returned with status %d\n"),
+			         Prog, editor, WEXITSTATUS (status));
+			exit (WEXITSTATUS (status));
+		} else if (WIFSIGNALED (status)) {
+			fprintf (stderr, _("%s: %s killed by signal %d\n"),
+			         Prog, editor, WTERMSIG (status));
+			exit (1);
 		} else {
 			exit (0);
 		}
 	}
 
+	/* Run child in a new pgrp and make it the foreground pgrp. */
+	if (orig_pgrp != -1) {
+		setpgid(pid, pid);
+		tcsetpgrp(STDIN_FILENO, pid);
+
+		/* Avoid SIGTTOU when changing foreground pgrp below. */
+		sigemptyset(&mask);
+		sigaddset(&mask, SIGTTOU);
+		sigprocmask(SIG_BLOCK, &mask, &omask);
+	}
+
+	/* set SIGCHLD to default for waitpid */
+	signal(SIGCHLD, SIG_DFL);
+
 	for (;;) {
 		pid = waitpid (pid, &status, WUNTRACED);
 		if ((pid != -1) && (WIFSTOPPED (status) != 0)) {
 			/* The child (editor) was suspended.
-			 * Suspend vipw. */
+			 * Restore terminal pgrp and suspend vipw. */
+			if (orig_pgrp != -1) {
+				editor_pgrp = tcgetpgrp(STDIN_FILENO);
+				if (editor_pgrp == -1) {
+					fprintf (stderr, "%s: %s: %s", Prog,
+						 "tcgetpgrp", strerror (errno));
+				}
+				if (tcsetpgrp(STDIN_FILENO, orig_pgrp) == -1) {
+					fprintf (stderr, "%s: %s: %s", Prog,
+						 "tcsetpgrp", strerror (errno));
+				}
+			}
 			kill (getpid (), SIGSTOP);
 			/* wake child when resumed */
-			kill (pid, SIGCONT);
+			if (editor_pgrp != -1) {
+				if (tcsetpgrp(STDIN_FILENO, editor_pgrp) == -1) {
+					fprintf (stderr, "%s: %s: %s", Prog,
+						 "tcsetpgrp", strerror (errno));
+				}
+			}
+			killpg (pid, SIGCONT);
 		} else {
 			break;
 		}
 	}
 
-	if (   (-1 == pid)
-	    || (WIFEXITED (status) == 0)
-	    || (WEXITSTATUS (status) != 0)) {
+	if (orig_pgrp != -1)
+		sigprocmask(SIG_SETMASK, &omask, NULL);
+
+	if (-1 == pid) {
 		vipwexit (editor, 1, 1);
+	} else if (   WIFEXITED (status)
+	           && (WEXITSTATUS (status) != 0)) {
+		vipwexit (NULL, 0, WEXITSTATUS (status));
+	} else if (WIFSIGNALED (status)) {
+		fprintf (stderr, _("%s: %s killed by signal %d\n"),
+		         Prog, editor, WTERMSIG(status));
+		vipwexit (NULL, 0, 1);
 	}
 
 	if (stat (fileedit, &st2) != 0) {
@@ -338,7 +392,7 @@ vipwedit (const char *file, int (*file_lock) (void), int (*file_unlock) (void))
 #ifdef WITH_SELINUX
 	/* unset the fscreatecon */
 	if (is_selinux_enabled () != 0) {
-		if (setfscreatecon (NULL) != 0) {
+		if (setfscreatecon_raw (NULL) != 0) {
 			vipwexit (_("setfscreatecon () failed"), errno, 1);
 		}
 	}
@@ -418,6 +472,8 @@ int main (int argc, char **argv)
 	bool do_vipw;
 
 	Prog = Basename (argv[0]);
+	log_set_progname(Prog);
+	log_set_logfd(stderr);
 
 	(void) setlocale (LC_ALL, "");
 	(void) bindtextdomain (PACKAGE, LOCALEDIR);
@@ -480,6 +536,10 @@ int main (int argc, char **argv)
 				usage (E_USAGE);
 			}
 		}
+
+		if (optind != argc) {
+			usage (E_USAGE);
+		}
 	}
 
 	if (do_vipw) {
@@ -533,6 +593,7 @@ int main (int argc, char **argv)
 
 	nscd_flush_cache ("passwd");
 	nscd_flush_cache ("group");
+	sssd_flush_cache (SSSD_DB_PASSWD | SSSD_DB_GROUP);
 
 	return E_SUCCESS;
 }
